#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   main.py
@Time    :   2022/08/10 11:10:06
@Author  :   ykzhou 
@Version :   0.0
@Contact :   ykzhou@stu.xidian.edu.cn
@Desc    :   None
'''

from random import seed
from Kmeans import Kmeans
import os
import argparse 

parser = argparse.ArgumentParser(description="hyper parameters for kmeans")

parser.add_argument("ImageName",type=str,help="select the image file name, must in the example directory")
parser.add_argument("-k",type=int,default=4,help="number of center")
parser.add_argument("--MaxIter",type=int,default=30,help="max iterations of kmeans algorithm")
parser.add_argument("--seed",type=int,default=0,help="seed for random center")

args = parser.parse_args()

# hyper parameters
imageName = args.ImageName
resultPath = os.getcwd() + '/result'
k = args.k
max_iter = args.MaxIter
seed = args.seed

solutions = Kmeans(k,resultPath,max_iter=max_iter)

if __name__ == "__main__":
    solutions.run(imageName,seed=seed)


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   __init__.py
@Time    :   2022/08/17 10:43:34
@Author  :   ykzhou 
@Version :   0.0
@Contact :   ykzhou@stu.xidian.edu.cn
@Desc    :   None
'''


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   Kmeans.py
@Time    :   2022/08/10 11:10:16
@Author  :   ykzhou 
@Version :   0.0
@Contact :   ykzhou@stu.xidian.edu.cn
@Desc    :   None
'''

from sys import flags
import numpy as np
import cv2 
import os

# The role of this class is to call the Kmeans algorithm on the image
class Kmeans(object):
    def __init__(self, k, resultPath, max_iter=25):
        self.k = k
        self.resultPath = resultPath
        self.max_iter = max_iter
        self.flags = cv2.KMEANS_RANDOM_CENTERS
        self.criteria = (cv2.TermCriteria_EPS+cv2.TermCriteria_MAX_ITER,max_iter,0.1)

    def setk(self, k):
        self.k = k
        
    def setResultPath(self, path):
        self.resultPath = path
    
    def readImage(self, imagePath):
        imagePath = os.getcwd() + '/example/' + imagePath
        print("input image...")
        img = cv2.imread(imagePath)

        return img
    
    def model(self, image, seed):
        np.random.seed(seed)
        img = image.reshape((-1, 3)).astype(np.float32)
        r, best, center = cv2.kmeans(img, self.k, None, criteria=self.criteria,
                                        attempts=10, flags=self.flags)
        center = np.uint8(center)
        # print(img[np.where(best==2)[0]])
        for i in range(self.k):
            img[np.where(best==i)[0]] = center[i]
        segmentation_img = img.reshape(image.shape)
        return segmentation_img
    
    def run(self, imageName, seed=0):
        image = self.readImage(imageName)
        result = self.model(image, seed)  
        cv2.imwrite(self.resultPath+'\\'+imageName , result)
        print("Successfully generated")


        


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name= "SegbyKmeans", 
    version = "1.0", 
    author="YuKun Zhou", 
    author_email="Yu-Kun-Zhou@outlook.com", 
    description="This is my first packages", 
    long_description=long_description, 
    long_description_content_type="text/markdown", 
    packages=setuptools.find_packages(), 
    
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    
    install_requires=[
        'opencv-python',
    ],
    python_requires=">=3",
    # url="https://github.com/1zeryu/Image-Segmentation", # github地址
)